@@ -1,4 +1,5 @@
 import asyncio
+import ssl
 from typing import Optional
 
 from . import schema
@@ -13,9 +14,15 @@ class ConnectionClosed(Exception):
 
 
 class Connection:
-    def __init__(self, host: str, port: int) -> None:
+    def __init__(
+        self,
+        host: str,
+        port: int,
+        ssl_context: Optional[ssl.SSLContext] = None,
+    ) -> None:
         self.host = host
         self.port = port
+        self._ssl_context = ssl_context
 
         self._buffer = bytearray()
         self._reader: Optional[asyncio.StreamReader] = None
@@ -27,11 +34,12 @@ class Connection:
                 asyncio.open_connection(
                     host=self.host,
                     port=self.port,
+                    ssl=self._ssl_context,
                 ),
                 timeout=CONNECT_TIMEOUT,
             )
         except asyncio.TimeoutError:
-            raise ConnectionError(f'Could not connect to {self.host}:{self.port}')
+            raise ConnectionError(f"Could not connect to {self.host}:{self.port}")
 
     async def close(self) -> None:
         assert self._writer is not None
@@ -55,7 +63,7 @@ class Connection:
         await self._writer.drain()
 
     async def _read_frame_raw(self) -> bytearray:
-        length = int.from_bytes(await self._read(4), 'big')
+        length = int.from_bytes(await self._read(4), "big")
         return await self._read(length)
 
     async def write_frame(self, frame: schema.Frame) -> None:
