@@ -81,3 +81,7 @@ class PreconditionFailed(ServerError):
 
 class PublisherDoesNotExist(ServerError):
     code = 18
+
+
+class OffsetNotFound(ServerError):
+    code = 19
