@@ -1,7 +1,11 @@
+# Copyright 2023 VMware, Inc. All Rights Reserved.
+# SPDX-License-Identifier: MIT
+
 import zlib
 from dataclasses import dataclass, field
 from typing import ClassVar, Optional, Type, cast
 
+from .compression import CompressionHelper, CompressionType
 from .constants import Key, OffsetType, T
 from .exceptions import ServerError
 
@@ -29,9 +33,9 @@ class Frame(Struct):
         assert cls.key is not NotImplemented
         registry[(is_response, cls.key)] = cls
 
-    def check_response_code(self) -> None:
+    def check_response_code(self, raise_exception: bool = True) -> None:
         code: int = getattr(self, "response_code", 0)
-        if code > 1:
+        if code > 1 and raise_exception is True:
             raise ServerError.from_code(code)
 
 
@@ -179,6 +183,19 @@ class Message(Struct):
     data: bytes = field(metadata={"type": T.bytes})
 
 
+@dataclass
+class PublishSubBatching(Frame):
+    key = Key.Publish
+    publisher_id: int = field(metadata={"type": T.uint8})
+    number_of_root_messages: int = field(metadata={"type": T.int32})
+    publishing_id: int = field(metadata={"type": T.uint64})
+    compress_type: int = field(metadata={"type": T.uint8})
+    subbatching_message_count: int = field(metadata={"type": T.uint16})
+    uncompressed_data_size: int = field(metadata={"type": T.int32})
+    compressed_data_size: int = field(metadata={"type": T.int32})
+    messages: bytes = field(metadata={"type": T.raw})
+
+
 @dataclass
 class Publish(Frame):
     key = Key.Publish
@@ -235,10 +252,10 @@ class MetadataResponse(Frame, is_response=True):
     brokers: list[Broker]
     metadata: list[StreamMetadata]
 
-    def check_response_code(self) -> None:
+    def check_response_code(self, raise_exception: bool = True) -> None:
         for item in self.metadata:
             code = item.response_code
-            if code > 1:
+            if code > 1 and raise_exception is True:
                 raise ServerError.from_code(code)
 
 
@@ -372,6 +389,39 @@ class QueryOffsetResponse(Frame, is_response=True):
     offset: int = field(metadata={"type": T.uint64})
 
 
+@dataclass
+class SubEntryChunk:
+    @classmethod
+    def read(self, data: bytes, entry_type: int, offset: int) -> tuple[list[bytes], int]:
+        # total bytes read
+        index = 0
+        num_records_in_batch = int.from_bytes(data[index + offset : index + offset + 2], byteorder="big")
+        index += 2
+        uncompressed_data_size = int.from_bytes(data[index + offset : index + offset + 4], byteorder="big")
+        index += 4
+        data_len = int.from_bytes(data[index + offset : index + offset + 4], byteorder="big")
+        index += 4
+
+        compression_type = CompressionType((entry_type & 0x70) >> 4)
+
+        current_pos = index + offset
+        data_compressed = data[current_pos : current_pos + data_len]
+        index += data_len
+
+        uncompressed_data = CompressionHelper.uncompress(
+            data_compressed, compression_type=compression_type, uncompressed_data_size=uncompressed_data_size
+        )
+        subbatch_entries = []
+        uncompressed_index = 0
+        for i in range(num_records_in_batch):
+            size = int.from_bytes(uncompressed_data[uncompressed_index : uncompressed_index + 4], "big")
+            uncompressed_index += 4
+            subbatch_entries.append(uncompressed_data[uncompressed_index : uncompressed_index + size])
+            uncompressed_index += size
+
+        return subbatch_entries, index
+
+
 @dataclass
 class Deliver(Frame):
     key = Key.Deliver
@@ -402,14 +452,21 @@ class Deliver(Frame):
     def get_messages(self) -> list[bytes]:
         messages = []
         pos = 0
+
         for _ in range(self.num_entries):
-            if (self.data[pos] & 0x80) == 0:
+            entry_type = self.data[pos]
+
+            if entry_type & 0x80 == 0:
                 size = int.from_bytes(self.data[pos : pos + 4], "big")
                 pos += 4
                 messages.append(self.data[pos : pos + size])
                 pos += size
+            # is a subentry-batch message
             else:
-                raise NotImplementedError
+                pos += 1
+                sub_batched_messages, total_bytes_read = SubEntryChunk.read(self.data, entry_type, pos)
+                messages.extend(sub_batched_messages)
+                pos += total_bytes_read
 
         return messages
 
@@ -426,3 +483,36 @@ class CreditResponse(Frame, is_response=True):
     key = Key.Credit
     response_code: int = field(metadata={"type": T.uint16})
     subscription_id: int = field(metadata={"type": T.uint8})
+
+
+# For new super stream implementation
+@dataclass
+class SuperStreamRoute(Frame):
+    key = Key.Route
+    # version : int = field(metadata={"type": T.uint16})
+    correlation_id: int = field(metadata={"type": T.uint32})
+    routing_key: str = field(metadata={"type": T.string})
+    super_stream: str = field(metadata={"type": T.string})
+
+
+@dataclass
+class SuperStreamRouteResponse(Frame, is_response=True):
+    key = Key.Route
+    correlation_id: int = field(metadata={"type": T.uint32})
+    response_code: int = field(metadata={"type": T.uint16})
+    streams: list[str] = field(metadata={"type": [T.string]})
+
+
+@dataclass
+class SuperStreamPartitions(Frame):
+    key = Key.Partitions
+    correlation_id: int = field(metadata={"type": T.uint32})
+    super_stream: str = field(metadata={"type": T.string})
+
+
+@dataclass
+class SuperStreamPartitionsResponse(Frame, is_response=True):
+    key = Key.Partitions
+    correlation_id: int = field(metadata={"type": T.uint32})
+    response_code: int = field(metadata={"type": T.uint16})
+    streams: list[str] = field(metadata={"type": [T.string]})
